# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime
import xml.etree.cElementTree as ET

from driverng import MessageV2, utils, Worklist


class Driver(MessageV2):
    SPEC = {"code": "iq200xml", "label": "Iq200 XML"}

    def parse(self, data):
        return ET.fromstring(
            data.replace("\u2019", "'").replace("\x19", "'").encode("utf16")
        )

    def to_string(self, message):
        header = '<?xml version="1.0" encoding="UTF-16"?>\r\n'
        return header + ET.tostring(message).decode()

    def encode_worklist(self, worklist, context):
        message = ET.Element("SI")
        message.set("BF", "URN")
        message.set("WO", "run")
        has_order = False
        for sample in worklist.samples:
            message.text = sample.sample_id
            demog = ET.SubElement(message, "DEMOG")
            for order in sample.orders:
                # Les suppressions ne sont pas gérées
                if order.action == "del":
                    continue
                has_order = True
                demog.set("DOB", utils.format_date(sample.birthdate, fmt="%Y-%m-%d"))
                demog.set("FName", sample.first_name)
                demog.set("Gender", sample.sex)
                demog.set("LName", sample.last_name)
                demog.set("RecNum", sample.patient_id)
        if has_order:
            return message

    def decode_message(self, message):
        if message.tag == "SIQ":
            worklist = Worklist(context="query")
            worklist.get_or_create(sample_id=message.text, type="patient")
        else:
            worklist = Worklist(context="result")

            # Internal QC
            if message.tag == "CQC":
                sample = worklist.get_or_create(
                    type="control",
                    sample_id=message.get("LOT"),
                    control_lot=message.get("LOT"),
                )
                sample.results.add(
                    code=message.get("PRODID"),
                    value=message.find("COUNT").text,
                    date=datetime.datetime.strptime(
                        message.get("RDTS"), "%Y-%m-%d %H:%M:%S"
                    ),
                )

            if message.tag == "SA":
                cristal = set()
                cylinder = set()
                other_cells = set()
                date = datetime.datetime.strptime(
                    message.get("RDTS"), "%Y-%m-%d %H:%M:%S"
                )
                dilution = message.get("DILD")
                sample_id = message.get("ID")

                # External QC
                if sample_id.startswith("UR"):
                    sample = worklist.get_or_create(
                        type="control", sample_id=sample_id, control_lot=sample_id
                    )
                    for ac in message.findall("AC"):
                        if ac.get("AT") == "Sediment":
                            for ar in ac.findall("AR"):
                                test = ar.get("Key")
                                result = ar.text.split(" ")
                                if test in ("HYAL", "EPIC", "WBCT", "RBCT", "GRAN"):
                                    if result[0] == "[none]":
                                        cylinder.add(result[0])
                                    else:
                                        cylinder.add("%s%s" % (test, result[0]))
                                elif test in (
                                    "CACB",
                                    "CAPH",
                                    "CAOX",
                                    "TPO4",
                                    "URIC",
                                    "LEUC",
                                    "CYST",
                                    "TYRO",
                                    "AMOR",
                                    "UNCX",
                                ):
                                    if result[0] == "[none]":
                                        cristal.add(result[0])
                                    else:
                                        cristal.add("%s%s" % (test, result[0]))
                                if result[0] == "[none]":
                                    continue
                                sample.results.add(
                                    code=test, value=result[0], date=date
                                )
                    for test, result in (("CYLINDRE", cylinder), ("CRISTAL", cristal)):
                        if not len(result):
                            continue
                        elif len(result) > 1 and "[none]" in result:
                            result.remove("[none]")
                        sample.results.add(
                            code=test, value=";".join(sorted(result)), date=date
                        )
                # Patient
                else:
                    sample = worklist.get_or_create(type="patient", sample_id=sample_id)
                    for ac in message.findall("AC"):
                        if ac.get("AT") == "Sediment" or ac.get("AT") == "Chemistry":
                            for ar in ac.findall("AR"):
                                test = ar.get("Key")
                                result = ar.text.split(" ")
                                if test in ("HYAL", "EPIC", "WBCT", "RBCT", "GRAN"):
                                    if result[0] == "[none]":
                                        cylinder.add(result[0])
                                    else:
                                        cylinder.add("%s%s" % (test, result[0]))
                                elif test in (
                                    "CACB",
                                    "CAPH",
                                    "CAOX",
                                    "TPO4",
                                    "URIC",
                                    "LEUC",
                                    "CYST",
                                    "TYRO",
                                    "AMOR",
                                    "UNCX",
                                ):
                                    if result[0] == "[none]":
                                        cristal.add(result[0])
                                    else:
                                        cristal.add("%s%s" % (test, result[0]))
                                elif test in ("NSE", "WBCC", "BYST", "TRCH", "SPRM"):
                                    if result[0] == "[none]":
                                        other_cells.add(result[0])
                                    else:
                                        other_cells.add("%s%s" % (test, result[0]))
                                if test in ("COL", "CLA"):
                                    result = ar.text
                                else:
                                    result = result[0]
                                sample.results.add(
                                    code=test, value=result, dilution=dilution
                                )
                    for test, result in (
                        ("CYLINDRE", cylinder),
                        ("CRISTAL", cristal),
                        ("AUTRES", other_cells),
                    ):
                        if not len(result):
                            continue
                        elif len(result) > 1 and "[none]" in result:
                            # Plus d'un resultat != [none]
                            result.remove("[none]")
                        sample.results.add(
                            code=test, value=";".join(sorted(result)), dilution=dilution
                        )
                    if message.get("SQN"):
                        sample.results.add(code="SEQUENCE", value=message.get("SQN"))

        return worklist
