# -*- coding: utf8 -*-
from __future__ import unicode_literals

import logging

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    HprimDate,
    HprimForeign,
    HprimString,
    HprimList,
    HprimMultiple,
    Segment,
)
from driverng.astm.parser import RecursiveList
from driverng.utils import num_years

logger = logging.getLogger("driverng.drivers.remisol_v2")


class RemisolV2SegmentH(HeaderSegment):
    sender_name = HprimString((5, 1))
    device_id = HprimString((5, 2))
    receiver_id = HprimString((10, 1))
    special_instruction = HprimString(11)
    processing_id = HprimString(12, required=True, default="P")
    version = HprimString(13, required=True, default="1")
    date = HprimDate(14)

    patients = HprimForeign("P", "header")
    orders = HprimForeign("O", "header")
    queries = HprimForeign("Q", "header")
    infos = HprimForeign("M", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class RemisolV2SegmentP(Segment):
    laboratory_patient_id = HprimString(4)
    patient_id = HprimString(5)
    last_name = HprimString((6, 1))
    first_name = HprimString((6, 2))
    maiden_name = HprimString(7)
    birthdate = HprimDate((8, 1), fmt_date="%Y%m%d")
    age = HprimString((8, 2))
    age_unit = HprimString((8, 3))
    sex = HprimString(9)
    adress = HprimString(11)
    phone_number = HprimString(13)
    attending_physician = HprimString(14)
    height = HprimString(17)
    weight = HprimString(18)
    diagnosis = HprimString(19)
    medication = HprimString(20)
    diet = HprimString(21)
    admission_date = HprimDate(24)
    admission_status = HprimString(25)
    location = HprimString(26)

    orders = HprimForeign("O", "patient")


class RemisolV2SegmentO(Segment):
    sample_id = HprimString((3, 1))
    rack = HprimString((3, 2))
    position = HprimString((3, 3))
    control_lot = HprimString((3, 4))
    control_name = HprimString((3, 6))
    tests = HprimList(5, basefield=HprimMultiple(0, code=HprimString(4)))
    priority = HprimString(6)
    collection_date = HprimDate(8)
    action_code = HprimString(12)
    specimen_type = HprimString((16, 1))
    specimen_descriptor = HprimString((16, 2))
    physician = HprimString(17)

    results = HprimForeign("R", "order")


class RemisolV2SegmentR(Segment):
    test_code = HprimString((3, 4))
    reagent_lot = HprimString((3, 6))
    reagent_serial = HprimString((3, 7))
    value = HprimString((4, 1))
    value_points = HprimList(
        4, basefield=HprimMultiple(0, x_value=HprimString(1), y_value=HprimString(2))
    )
    interpretation = HprimString((4, 2))
    units = HprimString((5, 1))
    units_2 = HprimString((5, 2))
    abnormal_flags = HprimString(7)
    result_status = HprimString(9)
    operator = HprimString(11)
    complete_date = HprimDate(13)
    instrument = HprimString(14)


class RemisolV2SegmentC(Segment):
    source = HprimString(3)
    text = HprimString(4)


class RemisolV2SegmentQ(Segment):
    specimen_id = HprimString((3, 2))
    request_status = HprimString(13)


class RemisolV2SegmentL(Segment):
    termination_code = HprimString(3)


class RemisolV2SegmentM(Segment):
    record_id = HprimString(3)


class RemisolV2SegmentM101(Segment):
    record_id = HprimString(3)
    tube = HprimString(4)
    date = HprimDate(5)
    instrument = HprimString(6)


class RemisolV2SegmentM103(Segment):
    record_id = HprimString(3)
    tube = HprimString((4, 1))
    initial_volume = HprimString((4, 2))
    remain_volume = HprimString((4, 3))
    aliquot = HprimString((5, 1))
    requested_volume = HprimString((5, 2))
    tube_status = HprimString(6)
    tube_error = HprimString(7)
    date = HprimDate(8)
    track_line = HprimString(9)
    instrument = HprimString(10)
    unit = HprimString(11)
    position = HprimString(12)
    location = HprimString(13)
    shelf = HprimString(14)
    tray_id = HprimString((15, 1))
    workstation = HprimString((15, 2))
    tray_status = HprimString(16)


class Driver(AstmMessageV2):
    SPEC = {"code": "remisol_v2", "label": "Remisol V2 Advance"}
    SEGMENT_CLASS_MAP = {
        "H": RemisolV2SegmentH,
        "P": RemisolV2SegmentP,
        "O": RemisolV2SegmentO,
        "R": RemisolV2SegmentR,
        "C": RemisolV2SegmentC,
        "Q": RemisolV2SegmentQ,
        "L": RemisolV2SegmentL,
        "M": RemisolV2SegmentM,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    OPTIONS = [
        AstmMessageV2.Option(
            "remisol_v2_device_id",
            label="Paramétrer le contenu du champ Device ID du segment H",
            default="LIS",
        ),
        AstmMessageV2.Option(
            "remisol_v2_sender_name",
            label="Paramétrer le nom de l'émetteur du segment H",
            default="MCA",
        ),
        AstmMessageV2.Option(
            "remisol_v2_receiver_id",
            label="Paramétrer le code du récepteur du segment H",
            default="Remisol",
        ),
    ]

    def __init__(self, **kwargs):
        self.remisol_v2_device_id = kwargs.pop("remisol_v2_device_id", "LIS")
        self.remisol_v2_sender_name = kwargs.pop("remisol_v2_sender_name", "MCA")
        self.remisol_v2_receiver_id = kwargs.pop("remisol_v2_receiver_id", "Remisol")
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        message = RemisolV2SegmentH(type_segment="H", **kwargs)
        message.end = RemisolV2SegmentL(type_segment="L")
        return message

    def parse_segment(self, line, separators):
        segment = super(Driver, self).parse_segment(line, separators)
        if segment.type_segment == "M":
            if segment.record_id not in ("101", "103"):
                logger.info(
                    "Segment M , dont le record id = {}, n'est pas traité".format(
                        segment.record_id
                    )
                )
            else:
                if segment.record_id == "103":
                    seg_m = RemisolV2SegmentM103()
                else:
                    seg_m = RemisolV2SegmentM101()
                splitted = RecursiveList.split(line, separators)
                seg_m.populate(splitted)
                return seg_m

        return segment

    def encode_worklist(self, worklist, context):
        msg = self.create(
            device_id=self.remisol_v2_device_id,
            date=context["now"],
            sender_name=self.remisol_v2_sender_name,
            receiver_id=self.remisol_v2_receiver_id,
        )
        if not msg.sender_name:
            msg.sender_name = worklist.sender_id
        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority"],
            action={"del": "C", "add": "A", "new": "A"},
            priority={"S": "S", worklist.ELSE: "R"},
        ):
            age = num_years(patient.birthdate, patient.collection_date)
            unit_age = "Y"
            seg_p = msg.patients.add(
                laboratory_patient_id=patient.patient_id,
                last_name=patient.last_name,
                first_name=patient.first_name,
                mother_maiden_name=patient.maiden_name,
                birthdate=patient.birthdate,
                sex=patient.sex,
                age=age,
                unit_age=unit_age,
                location=patient.location,
            )
            physician = [
                p for p in patient.physicians if p.type.lower() == "prescriber"
            ]
            for sample, action, priority in patient.samples:
                seg_o = seg_p.orders.add(
                    collection_date=sample.collection_date,
                    priority=priority,
                    action_code=action,
                    specimen_type=sample.specimen,
                )
                seg_o.sample_id = sample.sample_id
                seg_p.attending_physician = physician[0].last_name if physician else ""
                for order in sample.orders:
                    t = seg_o.tests.add()
                    t.code = order.code
                for result in sample.results:
                    seg_o.results.add(
                        test_code=result.code,
                        value=result.value,
                        units=result.unit,
                        result_status="R",
                        complete_date=result.date,
                    )

        if msg.patients:
            return msg

    def decode_message(self, message):
        if message.queries:
            worklist = Worklist(context="query")
            for seg_q in message.queries:
                worklist.get_or_create(type="patient", sample_id=seg_q.specimen_id)
            return worklist

        worklist = Worklist(context="result")
        for seg_p in message.patients:
            for seg_o in seg_p.orders:
                rack = "{}_{}".format(seg_o.position, seg_o.rack)
                if seg_o.action_code == "Q":
                    sample = worklist.get_or_create(
                        sample_id=seg_o.control_name,
                        control_lot=seg_o.control_lot,
                        type="control",
                    )
                else:
                    sample = worklist.get_or_create(
                        sample_id=seg_o.sample_id, type="patient", rack=rack
                    )
                for seg_r in seg_o.results:
                    if "(" in seg_r.value:
                        left_index = seg_r.value.find(" (") + 2
                        right_index = seg_r.value.find(")")
                        interpretation_flags = seg_r.value[
                            left_index:right_index
                        ].split()
                        value = seg_r.value[: left_index - 2]
                        flags = interpretation_flags
                        if seg_r.abnormal_flags:
                            flags.append(seg_r.abnormal_flags)
                    else:
                        if seg_r.interpretation:
                            interpretations = seg_r.interpretation.split()
                            flags = interpretations
                            if seg_r.abnormal_flags:
                                flags.append(seg_r.abnormal_flags)
                        else:
                            flags = [seg_r.abnormal_flags]
                        value = seg_r.value
                    comments = []
                    for seg_c in seg_r.commentaires:
                        if seg_c.source == "I" or seg_c.source == "R":
                            for c in seg_c.text.split(";"):
                                flags.append(c)
                        else:
                            for c in seg_c.text.split(";"):
                                comments.append(c)
                    # pour gérer les unités du type 10^3mm3
                    if seg_r.units_2:
                        unit = "{}^{}".format(seg_r.units, seg_r.units_2)
                    else:
                        unit = seg_r.units
                    if not seg_r.units == "XYCOORD":
                        sample.results.add(
                            code=seg_r.test_code,
                            date=seg_r.complete_date,
                            value=value,
                            comments=comments,
                            flags=flags,
                            instrument=seg_r.instrument,
                            unit=unit,
                            operator=seg_r.operator,
                        )
                    else:
                        points = []
                        for a in seg_r.value_points:
                            points.extend((a.x_value, a.y_value))
                        sample.results.add(
                            code=seg_r.test_code,
                            value="GRAPHE",
                            graph={"points": points},
                        )
                for seg_c in seg_o.commentaires:
                    sample.results.add(
                        code="COM%s" % (seg_c.rang_segment,),
                        date=message.date,
                        value=seg_c.text,
                    )

        if message.infos:
            for seg_m in message.infos:
                sample = worklist.get_or_create(type="patient", sample_id=seg_m.tube)
                if seg_m.record_id == "101":
                    sample.results.add(
                        code="TUBE ARRIVAL", value=seg_m.instrument, date=seg_m.date
                    )
                if seg_m.record_id == "103":
                    sample.rack = seg_m.tray_id
                    sample.results.add(
                        code="TUBE TRACKING", value=seg_m.instrument, date=seg_m.date
                    )
                    if seg_m.initial_volume:
                        sample.results.add(
                            code="Initial Vol", value=seg_m.initial_volume
                        )
                    if seg_m.remain_volume:
                        sample.results.add(code="Remain Vol", value=seg_m.remain_volume)
                    if seg_m.aliquot:
                        sample.results.add(code="Aliquot Tube ID", value=seg_m.aliquot)
                    if seg_m.requested_volume:
                        sample.results.add(
                            code="Requested Vol", value=seg_m.requested_volume
                        )
                    if seg_m.tube_status:
                        sample.results.add(code="Tube status", value=seg_m.tube_status)
                    if seg_m.tube_error and seg_m.tube_error != "0":
                        sample.results.add(code="Tube error", value=seg_m.tube_error)
                    if seg_m.track_line:
                        sample.results.add(code="Track Line ID", value=seg_m.track_line)
                    if seg_m.unit:
                        sample.results.add(code="Unit", value=seg_m.unit)
                    if seg_m.position:
                        sample.results.add(code="Position", value=seg_m.position)
                    if seg_m.location:
                        sample.results.add(code="Location", value=seg_m.location)
                    if seg_m.shelf:
                        sample.results.add(code="Shelf", value=seg_m.shelf)
                    if seg_m.workstation:
                        sample.results.add(
                            code="Workstation name", value=seg_m.workstation
                        )
                    if seg_m.tray_status:
                        sample.results.add(code="Tray status", value=seg_m.tray_status)

        return worklist
